package opendic;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.ViewPart;

public class DicView extends ViewPart {
	private Text text;
	private Browser browser;
	private Dictionary dictionary;
	private IActionBars bars;
	private SearchAction search;
	private IAction historyClear;
	private LinkedList<IAction> history = new LinkedList<IAction>();

	public DicView() {
		dictionary = (Dictionary) new EnglishDic();
		PopUpMenu.setInstance(this);
	}

	public void createPartControl(Composite parent) {

		parent.setLayout(new GridLayout(1, false));

		Group group = new Group(parent, SWT.NONE);
		group.setText("ܾ ˻");
		group.setLayout(new GridLayout(2, false));
		group.setLayoutData(new GridData(210, 60));

		text = new Text(group, SWT.SINGLE | SWT.BORDER);
		GridData gridData = new GridData(190, 15);
		gridData.horizontalSpan = 2;
		text.setLayoutData(gridData);
		text.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {
				if (e.keyCode == 13) {
					translate();
				}
			}

			public void keyReleased(KeyEvent e) {
			}
		});

		Button button = new Button(group, SWT.PUSH);
		button.setLayoutData(new GridData(70, 25));
		button.setText("search");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				translate();
			}
		});

		Button button2 = new Button(group, SWT.PUSH);
		button2.setLayoutData(new GridData(70, 25));
		button2.setText("clear");

		browser = new Browser(parent, SWT.MULTI | SWT.BORDER);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		browser.setLayoutData(gridData);
		browser.addLocationListener(new LocationListener() {
			public void changing(LocationEvent event) {
				if (event.location.indexOf("about:blank") < 0
						&& dictionary instanceof JapaneseDic) {
					String newUrl = "http://engdic.daum.net/"
							+ event.location.substring(6);
					browser.setText(dictionary.urlToHtml(newUrl));
				}
			}

			public void changed(LocationEvent event) {
			}
		});

		makeAction();

	}

	private void makeAction() {
		bars = getViewSite().getActionBars();
		search = new SearchAction();
		bars.getToolBarManager().add(search);

		search.setText("");
		search.setToolTipText("");

		historyClear = new Action() {
			public void run() {
				historyClear();
			}
		};
		historyClear.setText("");

		try {
			search.setImageDescriptor(getImageDescriptor("icons/tsearch_obj.gif"));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}

		bars.getToolBarManager().add(new Separator());

		IAction dialogAction = new Action() {
			public void run() {
				Shell shell = new Shell();
				MyDialog dialog = new MyDialog(shell);
				if (dialog.open() == IDialogConstants.OK_ID) {
					if (dialog.dicNum == 0)
						dictionary = new EnglishDic();
					if (dialog.dicNum == 1)
						dictionary = new JapaneseDic();
				}
			}
		};
		dialogAction.setToolTipText("");
		bars.getToolBarManager().add(dialogAction);
		try {
			dialogAction
					.setImageDescriptor(getImageDescriptor("icons/prop_ps.gif"));
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	private void historyClear() {
		search.clear();
	}

	public void translate() {
		String getText = text.getText();
		search.add(getText);
		browser.setText(dictionary.getHtml(getText));
	}

	public void translate(String word) {
		browser.setText(dictionary.getHtml(word));
	}
	
	public void popUpTranslate(String word){
		search.add(word);
		browser.setText(dictionary.getHtml(word));
	}

	public void setFocus() {
	}

	public static ImageDescriptor getImageDescriptor(String path)
			throws MalformedURLException {
		URL url = Activator.getDefault().getBundle().getEntry("");
		return ImageDescriptor.createFromURL(new URL(url, path));
	}

	class SearchAction extends Action implements IMenuCreator {

		Menu fMenu;

		SearchAction() {
			setMenuCreator(this);
		}

		public void run() {
			if (history.size() > 1)
				history.get(1).run();
		}

		public void dispose() {
		}

		public Menu getMenu(Control parent) {
			fMenu = new Menu(parent);
			for (int i = 0; i < history.size(); i++) {
				addActionToMenu(fMenu, history.get(i));
			}
			new MenuItem(fMenu, SWT.SEPARATOR);
			addActionToMenu(fMenu, search);
			addActionToMenu(fMenu, historyClear);
			return fMenu;
		}

		public Menu getMenu(Menu parent) {
			return null;
		}

		protected void addActionToMenu(Menu parent, IAction action) {
			ActionContributionItem item = new ActionContributionItem(action);
			item.fill(parent, -1);
		}

		public void add(final String word) {
			history.addFirst(new Action() {
				public void run() {
					translate(word);
				}
			});
			history.get(0).setText(word);
		}

		public void clear() {
			history.clear();
		}

	}

}
